/* Public libmudflap declarations -*- C -*- */

#ifndef MF_RUNTIME_H
#define MF_RUNTIME_H

#if 1 /* autoconf HAVE_STDINT_H */
#include <stdint.h>
#endif
#include <sys/types.h>
#if ! 1
typedef unsigned long uintptr_t;
#define HAVE_UINTPTR_T 1
/* Define this here, in case an autoconf application was run
   without CFLAGS=-fmudflap but is being compiled with -fmudflap.  */
#endif


/* Global declarations used by instrumentation. */

struct __mf_cache { uintptr_t low; uintptr_t high; };
extern struct __mf_cache __mf_lookup_cache [];
extern uintptr_t __mf_lc_mask;
extern unsigned char __mf_lc_shift;

/* Multithreading support.  */
#ifdef _MUDFLAPTH
/* extern pthread_mutex_t __mf_biglock; */
#endif

/* Codes to describe the type of access to check: __mf_check arg 3 */

#define __MF_CHECK_READ 0
#define __MF_CHECK_WRITE 1


/* Codes to describe a region of memory being registered: __mf_*register arg 3 */

#define __MF_TYPE_NOACCESS 0
#define __MF_TYPE_HEAP 1
#define __MF_TYPE_HEAP_I 2
#define __MF_TYPE_STACK 3
#define __MF_TYPE_STATIC 4
#define __MF_TYPE_GUESS 5


/* The public mudflap API */

#ifdef __cplusplus
extern "C" {
#endif

extern void __mf_check (void *ptr, size_t sz, int type, const char *location)
       __attribute((nothrow));
extern void __mf_register (void *ptr, size_t sz, int type, const char *name) 
       __attribute((nothrow));
extern void __mf_unregister (void *ptr, size_t sz) __attribute((nothrow));
extern unsigned __mf_watch (void *ptr, size_t sz);
extern unsigned __mf_unwatch (void *ptr, size_t sz);
extern void __mf_report ();
extern int __mf_set_options (const char *opts);


/* Redirect some standard library functions to libmudflap.  These are
   done by simple #define rather than linker wrapping, since only
   instrumented modules are meant to be affected.  */

#ifdef _MUDFLAP  
#define memcpy __mfwrap_memcpy 
#define memmove __mfwrap_memmove
#define memset __mfwrap_memset 
#define memcmp __mfwrap_memcmp 
#define memchr __mfwrap_memchr 
#define memrchr __mfwrap_memrchr
#define strcpy __mfwrap_strcpy 
#define strncpy __mfwrap_strncpy 
#define strcat __mfwrap_strcat 
#define strncat __mfwrap_strncat
#define strcmp __mfwrap_strcmp 
#define strcasecmp __mfwrap_strcasecmp 
#define strncmp __mfwrap_strncmp 
#define strncasecmp __mfwrap_strncasecmp
#define strdup __mfwrap_strdup 
#define strndup __mfwrap_strndup 
#define strchr __mfwrap_strchr 
#define strrchr __mfwrap_strrchr
#define strstr __mfwrap_strstr 
#define memmem __mfwrap_memmem 
#define strlen __mfwrap_strlen 
#define strnlen __mfwrap_strnlen
#define bzero __mfwrap_bzero 
#define bcopy __mfwrap_bcopy 
#define bcmp __mfwrap_bcmp 
#define index __mfwrap_index 
#define rindex __mfwrap_rindex
#define asctime __mfwrap_asctime
#define ctime __mfwrap_ctime
#define gmtime __mfwrap_gmtime
#define localtime __mfwrap_localtime

/* Disable glibc macros.  */
#define __NO_STRING_INLINES

#endif /* _MUDFLAP */


#ifdef __cplusplus
}
#endif

#endif /* MF_RUNTIME_H */
